var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const qiniu = __importStar(require("qiniu"));
const Utils_1 = require("./Utils");
const API_ROOT = process.env.SERVER || 'https://meshtech.co';
const SALT = 'iF0DcUu1xNj8IjTBIw02scEaZjCW1n0oYuRpWk9G';
/**
 * Deploy file to a qiniu bucket with certain key
 *
 * @param {string} filepath - path of the file to be deployed
 * @param {string} bucket - bucket we are deploying to
 * @param {string} key - key of the file appearing on bucket
 * @param {IEnvoyConfig} appConfig - app configuration for uploading
 * @param {string} fingerprint - fingerprint for uploading
 */
function deploy(filepath, bucket, key, appConfig, fingerprint) {
    return __awaiter(this, void 0, void 0, function* () {
        const config = new qiniu.conf.Config();
        config.zone = qiniu.zone.Zone_z2;
        // config.RPC_TIMEOUT = 60000 // 3 min.
        const uploader = new qiniu.form_up.FormUploader(config);
        const putExtra = new qiniu.form_up.PutExtra();
        const resultUrl = yield upload(uploader, putExtra, {
            filepath,
            bucket,
            key
        });
        return update(appConfig, resultUrl, fingerprint);
    });
}
exports.deploy = deploy;
/**
 * Performs upload via qiniu uploader
 *
 * @param {qiniu.form_up.FormUploader} uploader - the qiniu uploader
 * @param {qiniu.form_up.PutExtra} putExtra - the qiniu's PutExtra object
 * @param {IUploadOptions} opt - upload options
 */
function upload(uploader, putExtra, opt) {
    return __awaiter(this, void 0, void 0, function* () {
        const { filepath, bucket, key } = opt;
        const { uploadToken, downloadUrl } = yield getQiniuUploadToken(bucket, key);
        return new Promise((resolve, reject) => {
            putExtra.fname = key;
            putExtra.params = {
                'x:name': key
            };
            uploader.putFile(uploadToken, key, filepath, putExtra, function (respErr, respBody, respInfo) {
                if (respErr) {
                    console.log(respErr.name);
                    reject(respErr);
                }
                else if (respInfo.statusCode === 200) {
                    resolve(downloadUrl);
                }
                else {
                    console.log(respInfo.statusCode);
                    console.log(respBody);
                    reject(respBody);
                }
            });
        });
    });
}
/**
 * Update package information on MeshServer
 *
 * @param {IEnvoyConfig} appConfig - the envoy app config
 * @param {string} url - the updatd package url
 * @param {string} fingerprint - fingerprint for updating
 */
function update(appConfig, url, fingerprint) {
    return __awaiter(this, void 0, void 0, function* () {
        const data = {
            packageName: appConfig.name,
            downloadURL: url,
            fingerprint: fingerprint,
            meshEngineVersion: appConfig.engineVersion
        };
        const signature = 'sha1=' + Utils_1.createHmac(appConfig.privateKey, JSON.stringify(data), "base64");
        const server = API_ROOT + '/api/packages/update';
        // post to update package info
        const result = yield fetch(server, {
            method: 'POST',
            headers: {
                'X-Developer-Access-Key': appConfig.accessKey,
                'X-Mesh-Signature': signature,
                'Content-Type': 'application/json'
            },
            body: JSON.stringify(data)
        });
        // validate status
        if (result.status >= 400) {
            throw new Error(`mesh server return status code ${result.status}`);
        }
        // return result
        const res = yield result.json();
        if (res.code === 1) {
            return res.message;
        }
        else {
            throw new Error(`mesh server throw error ${res.message} when updating package`);
        }
    });
}
exports.update = update;
function getQiniuUploadToken(bucket, key) {
    return __awaiter(this, void 0, void 0, function* () {
        // pre-process the data
        const stringifyData = JSON.stringify({
            bucket,
            key,
            applyTime: Date.now()
        });
        const data = new Buffer(stringifyData).toString('base64');
        const param = data.replace(/\+/g, "-").replace(/\//g, "_");
        // get sign and url
        const sign = Utils_1.createHmac(SALT, param, "base64");
        const url = API_ROOT + '/api/v2/cdn/uploadToken';
        const result = yield fetch(url, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({
                sign: sign,
                param: param
            })
        });
        if (result.status >= 400) {
            throw new Error(`mesh server return status code ${result.status} when get upload token`);
        }
        const res = yield result.json();
        if (res.code === 1) {
            return res.data;
        }
        else {
            throw new Error(`mesh server throw error ${res.message} when get upload token`);
        }
    });
}
exports.getQiniuUploadToken = getQiniuUploadToken;
